#include "HPCxx_Comm.h"

int  BIND_HANDLER = 202;
int GF_HANDLER = 0; 

nexus_startpoint_t registry_sp;
nexus_startpoint_t skel_sp; // this is the sp for java skel obj
nexus_endpoint_t*  current_ep; // ep of the current context 
HPCxx_Sync<int> bind_sync;
HPCxx_Sync<int> lookup_sync;

////////////////////////////////////////////////////////////////////////////
// this procedure creates a binding between a URL object and the registry sp
////////////////////////////////////////////////////////////////////////////
void attach_to_registry(char *registryHost, unsigned short port){
  
  // creating a URL object with host and port as defined above
  char url[100];
  sprintf(url, "x-nexus://%s:%d", registryHost, port);
  printf("attach_to_registry(): URL = (%s:%d) \n",registryHost, port);
  char *host = strdup(registryHost);
  // binding the URL to the registry_sp
  nexus_printf("server-->in attach_to_registry(): attaching to url <%s:%d>\n", registryHost, port);
  
  //  int rc = globus_nexus_attach(url, &registry_sp);

  int rc  = hpcxx_grabContext(&registry_sp, host, port);
  
  // catching exception if it occurs
  if (rc != 0){
    nexus_printf("server-->attach_to_registry(): ERROR: failed with rc=%d\n", rc);
    //nexus_abort();
  }
  
}


////////////////////////////////////////////////////////////////////////////
// this function constructs and sends a message to the registry asking it 
// to register an object
////////////////////////////////////////////////////////////////////////////
template <class T>
void bind_registry(HPCxx_GlobalPtr<T> gp, char* rmiurl) {

  // get the startpoint of the current context...
  printf("server-->bind_registry(): getting startpoint of the current context. \n");
  nexus_startpoint_t localSP; 
  // startpoint for the object "Component"
  // which is actually a remote object for clients
  nexus_startpoint_t remoteObjSP;  

  nexus_endpoint_t*  hep;
  hep = hpcxx_get_ep(); 
  nexus_startpoint_bind(&localSP, hep);

  // get the startpoint of the user object....
  printf("server-->bind_registry(): getting startpoint of the user object. \n");
  HPCxx_GlobalRefImpl<T> &r = *gp;
  //  nexus_startpoint_t *osp = r.contextID_;
  nexus_startpoint_copy(&remoteObjSP, r.contextID_);

  // pack the buffer to be sent to the registry...
  printf("server-->bind_registry(): constructing message to be sent to registry\n");

  nexus_buffer_t sendBuffer;
  //  char *FQN = HPCxx_MyType<T>::getFullyQualifiedFileName();
  T *localPointer = gp;
  char *FQN = localPointer->getFullyQualifiedClassName();
  printf("\n HPCxx_comm.C:: \t FQN: %s ", FQN);
  char *serializationClass ="nexusrmi.Nexusize";
  char *codeBase = "";
  int objname_size = strlen(FQN);
  int url_size = strlen(rmiurl);  
  int serClass_size = strlen(serializationClass);
  int codeBase_size = strlen(codeBase);

  // initializing buffer with the right size
  nexus_buffer_init(&sendBuffer, 
		    nexus_sizeof_int(1) + // bind handler 
		    nexus_sizeof_byte(1) + // byte 2 for reply  sp
		    nexus_sizeof_startpoint(&localSP, 1) +
		    nexus_sizeof_byte(1) + // byte 2
		    nexus_sizeof_int(1) +  
		    nexus_sizeof_char(serClass_size) +
		    nexus_sizeof_byte(1) + // byte 0 for null codebase
		    nexus_sizeof_byte(1) + // byte 2 for sp of remote obj
		    nexus_sizeof_startpoint(&remoteObjSP, 1) +
		    nexus_sizeof_byte(1) + // byte 2
		    nexus_sizeof_int(1) +
		    nexus_sizeof_char(objname_size)+
		    nexus_sizeof_byte(1) + // byte 2
		    nexus_sizeof_int(1) +
		    nexus_sizeof_char(url_size),
		    0);
  // packing the buffer

  // 1. sp of the handler object, in this case the current context...

  //  nexus_byte_t byte_34 = 34;

  nexus_byte_t byte_2;
  unsigned char two = 2;
  byte_2 = two;

  nexus_byte_t byte_0;
  unsigned char zero = 0;
  byte_0 = zero;
 
  nexus_put_int(&sendBuffer, &BIND_HANDLER, 1);
  nexus_put_byte(&sendBuffer, &byte_2, 1);
  nexus_put_startpoint_transfer(&sendBuffer, (&localSP), 1);
  
  // 2. The serialization class...
  nexus_put_byte(&sendBuffer, &byte_2, 1);
  nexus_put_int(&sendBuffer, &serClass_size, 1);   
  nexus_put_char(&sendBuffer, serializationClass, serClass_size);

  // 3. The code base
  nexus_put_byte(&sendBuffer, &byte_0, 1);

  // 4. sp of the remote object...
  nexus_put_byte(&sendBuffer, &byte_2, 1);
  nexus_put_startpoint_transfer(&sendBuffer, &remoteObjSP, 1);

  // 3. obj name size
  nexus_put_byte(&sendBuffer, &byte_2, 1);
  nexus_put_int(&sendBuffer, &objname_size, 1);   

  // 4. obj name:  FQN
  nexus_put_char(&sendBuffer, FQN, objname_size);

  // 5. url size
  nexus_put_byte(&sendBuffer, &byte_2, 1);
  nexus_put_int(&sendBuffer, &url_size, 1);        

  // 6. url 
  nexus_put_char(&sendBuffer, rmiurl, url_size);     

  printf("\n server-->bind_registry(): sending buffer to registry\n");
  // problem: BIND_HANDLER defined as 3 in RMIRegistry.java
  // but 12 in hpcxx library.
  // for now: changed BIND_HANDLER to 12 in nexusrmi package

  nexus_send_rsr(&sendBuffer, &registry_sp,
		 GF_HANDLER, NEXUS_TRUE, NEXUS_TRUE);
  printf("\n server-->bind_registry(): send done\n");   
}  
  

/////////////////////////////////////////////////////////////////////////////////
//this function handles replies from the registry sent in response to a bind
//message
/////////////////////////////////////////////////////////////////////////////////
void receive_handler(nexus_endpoint_t *endpoint,
		     nexus_buffer_t *buffer,
		     nexus_bool_t is_non_threaded_handler) { 
  // receive an ack to the bind  message
  int returnHandler;
  nexus_get_int(buffer, &returnHandler, 1);
  switch(returnHandler) {
  case 202 : // FIXME, should be sth like case BIND_HANDLER
    printf("HPCxx_comm.C:: receive_handler(): Bind succeeded  ... \n");
    break;
    
  default:
    printf("HPCxx_comm.C:: Unknown returnHandler\n");
    break;
    
  }

}

void registry_handler(nexus_endpoint_t *endpoint,
		      nexus_buffer_t *buffer,
		      nexus_bool_t is_non_threaded_handler)
{
  // reply to lookup message
  // this method is not needed !!!
}











